import { co2 } from "https://esm.sh/@tgwf/co2@latest";

(function ($, Drupal, drupalSettings) {
  'use strict';
  Drupal.behaviors.carbonImpactButton = {
    attach: function (context) {
      $(once('bodycarbonbutton', '.carbon_impact_button')).click(function (e) {
        $(this).toggleClass('active');
        $(this).siblings().slideToggle();
        check_per_byte();
        check_per_visit();
      });
    }
  };

  /**
   * When the body has finished loading, it checks that
   * the Per Byte model is active and calculates the CO2
   * footprint per Byte of the current page.
   *
   * This footprint is expressed in gCO2/Per Byte.
   */
  function check_per_byte() {
    $(once('bodyloadperbyte', 'body')).each(function (e) {
      if (drupalSettings.per_byte == 1) {
        const oneByte = new co2({
          model: '1byte',
          results: 'segment',
          version: 4
        });
        let pageLoadSize = getPageSize();
        const greeHost = true;
        var perByte = oneByte.perByte(pageLoadSize, greeHost);

        // Format perByte to 3 decimal places.
        let perByteFormatted = parseFloat(perByte).toFixed(3);
        let perByteElement = $('body').find('.co2-per-byte .carbon_count');
        perByteElement.html(perByteFormatted);

        // And classes to parent to change the color of the values.
        let numberByteContainer = perByteElement.parent();
        if (parseFloat(perByteFormatted) <= 0.095) {
          numberByteContainer.addClass('a_plus');
        } else if (parseFloat(perByteFormatted) <= 0.186) {
          numberByteContainer.addClass('a');
        } else if (parseFloat(perByteFormatted) <= 0.341) {
          numberByteContainer.addClass('b');
        } else if (parseFloat(perByteFormatted) <= 0.493) {
          numberByteContainer.addClass('c');
        } else if (parseFloat(perByteFormatted) <= 0.656) {
          numberByteContainer.addClass('d');
        } else if (parseFloat(perByteFormatted) <= 0.846) {
          numberByteContainer.addClass('e');
        } else {
          numberByteContainer.addClass('f');
        }

        // Adjust with actual co2Level value
        updateCarbonLevelIndicator(parseFloat(perByteFormatted), $('.per_byte'));

        sendPervisitData(perByte, 'per_byte');
      }
    });
  }

  /**
   * When the body has finished loading, it checks that the Per Visit
   * model is active and calculates the CO2 footprint per Visit
   * of the current page.
   *
   * This footprint is expressed in gCO2/Per Visit.
   */
  function check_per_visit() {
    $(once('bodyloadpervisit', 'body')).each(function (e) {
      if (drupalSettings.per_visit == 1) {
        let nid = drupalSettings.nid;
        var visitStorage = localStorage.getItem(nid);
        var visitStatus = '';
        var reloadSize = '';
        if (visitStorage == null) {
          visitStatus = 'first_visit';
          localStorage.setItem(nid, visitStatus);
          let pageSize = getPageSize();
          localStorage.setItem(nid + '_size', pageSize);
          reloadSize = 0;
        } else {
          visitStatus = 'return_visit';
          localStorage.setItem(nid, visitStatus);
          reloadSize = getPageSize();
        }
        sendRequest(visitStatus, reloadSize, drupalSettings);
      }
    });
  }

  /**
   * It creates an Ajax request of the POST type which is then received in the module's backoffice.
   * This request sends the page ID and the status of the visit.
   *
   * @param {visitStatus} visitStatus - It checks whether it's the first time the user has visited that page or not.
   * @param {reloadSize} reloadSize - India the number of bytes when the page is reloaded.
   * @param {drupalSettings} drupalSettings - It mentions all the values stored in the Drupal settings.
   */
  function sendRequest(visitStatus, reloadSize, drupalSettings) {
    let nid = drupalSettings.nid;
    const hostname = location.hostname;
    $.ajax({
      url: 'https://' + hostname + '/carbon-impact-evaluator/visits',
      method: 'POST',
      data: {
        valor: JSON.stringify({
          nid: nid,
          visit_status: visitStatus
        })
      },
      success: function (response) {
        getCO2(response, reloadSize);
      },
      error: function (error) {
        // console.error(Drupal.t('Erro ao enviar o valor:'), error);
      }
    });
  }

  /**
   *
   * @returns The page size.
   */
  function getPageSize() {
    const resources = performance.getEntriesByType('resource');
    var resourcesSize = 0;
    resources.forEach((element) => {
      resourcesSize += element.transferSize;
    });
    const navigation = performance.getEntriesByType('navigation')[0];
    let navigationSize = navigation.transferSize;
    let totalSize = resourcesSize + navigationSize;
    return totalSize;
  }

  /**
   * It works on the response to the Ajax POST request created in the
   * function and calculates the carbon footprint.
   *
   * @param {response} response - Ajax POST request response.
   * @param {reloadSize} reloadSize - Number of bytes of the page reload.
   */
  function getCO2(response, reloadSize) {
    let nid = drupalSettings.nid;
    let first = response.valor.first_visit;
    let total = response.valor.total_number_visits;
    let returnV = response.valor.return_visit;
    // 2) Vamos fazer os cálculos
    let firstPercentage = parseFloat((first * 100) / total / 100);
    let returnPercentage = parseFloat((returnV * 100) / total / 100);
    // Uses the most recent version for this model (v0.16.1). The default is version 3 (v0.11.0).
    const carbonEmissions = new co2({
      model: 'swd',
      version: 4
    });
    let pageSize = localStorage.getItem(nid + '_size');
    var reloadRatio = (reloadSize * 100) / pageSize / 100;
    const greeHost = drupalSettings.Greenhost;
    let datacenterCountry = drupalSettings.Datacenter_country;
    const options = {
      dataReloadRatio: parseFloat(reloadRatio),
      firstVisitPercentage: parseFloat(firstPercentage),
      returnVisitPercentage: parseFloat(returnPercentage),
      gridIntensity: {
        dataCenter: { country: datacenterCountry }
      }
    };
    var calculatedCarbonEmissions = carbonEmissions.perVisitTrace(pageSize, greeHost, options).co2;

    // Format perVisit to 3 decimal places.
    let perVisitFormatted = parseFloat(calculatedCarbonEmissions).toFixed(3);
    let perVisitElement = $('body').find('.co2-per-visit .carbon_count');
    perVisitElement.html(perVisitFormatted);

    // And classes to parent to change the color of the values.
    let numberVisitContainer = perVisitElement.parent();
    if (parseFloat(perVisitFormatted) <= 0.095) {
      numberVisitContainer.addClass('a_plus');
    } else if (parseFloat(perVisitFormatted) <= 0.186) {
      numberVisitContainer.addClass('a');
    } else if (parseFloat(perVisitFormatted) <= 0.341) {
      numberVisitContainer.addClass('b');
    } else if (parseFloat(perVisitFormatted) <= 0.493) {
      numberVisitContainer.addClass('c');
    } else if (parseFloat(perVisitFormatted) <= 0.656) {
      numberVisitContainer.addClass('d');
    } else if (parseFloat(perVisitFormatted) <= 0.846) {
      numberVisitContainer.addClass('e');
    } else {
      numberVisitContainer.addClass('f');
    }

    // Adjust with actual co2Level value.
    updateCarbonLevelIndicator(parseFloat(perVisitFormatted), $('.per_visit'));

    sendPervisitData(calculatedCarbonEmissions, 'per_visit');
  }

  /**
   * Sends the calculated Per Visit values to the module's back office so that they can be stored in the table.
   *
   * @param {co2Calculations} co2Calculations - The result of the Per Visit/Per Byte calculations.
   * @param {identifier} identifier - Model identifier.
   */
  function sendPervisitData(co2Calculations, identifier) {
    let nid = drupalSettings.nid;
    const hostname = location.hostname;
    $.ajax({
      url: 'https://' + hostname + '/carbon-impact-evaluator/pervisit',
      method: 'POST',
      data: {
        valor: JSON.stringify({
          nid: nid,
          identifier: identifier,
          calculations: co2Calculations
        })
      },
      success: function (response) {
        // console.log(Drupal.t('Sucesso ao enviar o pedido:'), response);
      },
      error: function (error) {
        // console.error(Drupal.t('Erro ao enviar o valor:'), error);
      }
    });
  }

  /**
   * Changes the co2 level indicator acording to the level received from byte function and visit function.
   *
   */
  function updateCarbonLevelIndicator(co2Level, element) {
    const levels = [
      { max: 0.095, className: 'a_plus' },
      { max: 0.186, className: 'a' },
      { max: 0.341, className: 'b' },
      { max: 0.493, className: 'c' },
      { max: 0.656, className: 'd' },
      { max: 0.846, className: 'e' },
      { max: Infinity, className: 'f' }
    ];

    // Find the level corresponding to the co2Level.
    let currentLevel = levels.find((level) => co2Level <= level.max);
    if (!currentLevel) return;

    // Find the element corresponding to the current level.
    const currentLevelElement = element.find('.carbon_level_bar').find(`.${currentLevel.className}`);

    // Calculate the position for the arrow
    const levelPosition = currentLevelElement.position();
    const levelWidth = currentLevelElement.width();
    // Move the arrow to the center of the current level section
    element
      .find('.arrow_indicator')
      .css('left', levelPosition.left + levelWidth / 2 - $('.arrow_indicator').width() / 2);
  }
})(jQuery, Drupal, drupalSettings);
