<?php

declare(strict_types=1);

namespace Drupal\carbon_impact_evaluator\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Carbon impact evaluator settings.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'carbon_impact_evaluator_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['carbon_impact_evaluator.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('carbon_impact_evaluator.settings');
    $form['description'] = [
      '#type' => 'item',
      '#markup' => $this->t('Configuration form for the module "Carbon impact evaluator". This form should define the fixed values that should be taken into account when calculating carbon emissions, Per Visit and/or Per Byte.'),
    ];
    $form['models_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('CO2 calculation models'),
      '#description' => $this->t('Choose the CO2 calculation model(s) you want to implement.')
    ];
    $form['models_fieldset']['per_byte'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Per Byte'),
      '#default_value' => $this->config('carbon_impact_evaluator.settings')->get('per_byte'),
      '#description' => $this->t('Choose the CO2 calculation model(s) you want to implement.')
    ];
    $form['models_fieldset']['per_visit'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Per Visit'),
      '#default_value' => $this->config('carbon_impact_evaluator.settings')->get('per_visit'),
      '#description' => $this->t('Choose the CO2 calculation model(s) you want to implement.')
    ];
    $form['greenhost'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Green Host'),
      '#default_value' => $this->config('carbon_impact_evaluator.settings')->get('greenhost'),
      '#description' => $this->t('Field to indicate whether your website data is being stored on a green web host.')
    ];
    $form['dataCenter'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Datacenter country'),
      '#default_value' => $this->config('carbon_impact_evaluator.settings')->get('dataCenter'),
      '#description' => $this->t('ISO alpha-3 country code for the zone where your datacenter is located. Example: Portugal is PRT. <br> For more information see <a href="https://www.iso.org/obp/ui/#search">ISO Country Codes Collection.</a>'),
      '#states' => [
        'visible' => [
            ':input[name="per_visit"]' => [
              'checked' => TRUE
            ]
          ],
        'required' => [
          ':input[name="per_visit"]' => [
            'checked' => TRUE
            ]
          ]
      ]
    ];
    $fields = [
      'greenhost',
      'dataCenter',
      'per_byte',
      'per_visit'
    ];
    $form_state->set('array_fields', $fields);
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   * 
   * If the model Per Visit is enabled, validates if insert ISO value for country is valid.
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    if($form_state->getValue('per_visit') == 1) {
      $registered_country_codes = SettingsForm::getCountriesCodes($form_state->getValue('dataCenter'));
      if(!preg_match('/^[A-Z]{3}$/', $form_state->getValue('dataCenter'))) {
        $form_state->setErrorByName(
          'message',
          $this->t('The value of <a href="#edit-datacenter">Datacenter country</a>, it is not a ternary code.'),
        );
      } elseif($registered_country_codes === FALSE) {
        $form_state->setErrorByName(
          'message',
          $this->t('The value of <a href="#edit-datacenter">Datacenter country</a>, should be a valid ISO alpha-3 country code.'),
        );
      }
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * Gets the ISO values from /data/countries-codes.json and compares with the inserted value.
   */
  function getCountriesCodes($code) {
    // Vamos obter o caminho para o nosso modulo
    $module_path = \Drupal::service('extension.list.module')->getPath('carbon_impact_evaluator');
    // Vamos obter o conteúdo para o nosso ficheiro
    $countries_codes = json_decode(file_get_contents($module_path . '/data/countries-codes.json'), TRUE);
    // Vamos definir uma variável para armazenar True
    $found = FALSE;
    // Vamos percorrer os códigos registrados
    foreach($countries_codes as $values) {
      if($values['Alpha-3 code'] == $code) {
        $found = TRUE;
      }
    }
    return $found;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $config = $this->config('carbon_impact_evaluator.settings');
    $forms_fields = $form_state->get('array_fields');
    foreach($forms_fields as $field) {
      $config->set($field, $form_state->getValue($field));
    }
    $config->save();
    parent::submitForm($form, $form_state);
  }

}
